import Config from "../config";
import path from "path";
import { getLocale } from "next-intl/server";
import { Locale } from "@/i18n/config";

const baseUrl = Config.apiBaseUrl;

/**
 * Fetches data from the API endpoint using server-side methods.
 * Should be used in server components instead of `useFetchApi`.
 *
 * @param endpoint - The API endpoint to call
 * @param init - Optional fetch init options
 * @param version - The API version to use
 * @param queryParams - Optional query parameters
 * @param locale - The locale to use for localization. (if not provided fallsback to using `genericPageStore`)
 * @returns The API response
 */
export async function fetchApi(
  endpoint: string,
  {
    init,
    queryParams,
    locale,
  }: { locale?: Locale } & ServerRequestOptions = {},
) {
  if (!locale) {
    const lang = (await getLocale()) as Locale;
    locale = lang;
  }

  let requestOptions: RequestInit = init ?? {};
  if (!init?.next?.revalidate) {
    requestOptions = {
      ...init,
      next: {
        ...init?.next,
        revalidate: 60 * 5,
      },
    };
  }

  const url = generateApiUrl({ endpoint, queryParams, locale });
  const startTime = performance.now();
  const res = await fetch(url, requestOptions);
  const endTime = performance.now();
  console.info(
    "%s (status: %d, elapsed time: %fms)",
    url,
    res.status,
    (endTime - startTime).toFixed(2),
  );
  return res;
}

/**
 * Generates a URL for making API requests by combining the
 * base URL, API version, locale, endpoint and query parameters.
 *
 * @param endpoint - The API endpoint
 * @param version - The API version
 * @param locale - The locale to use
 * @param queryParams - Optional query parameters
 * @returns The constructed URL
 */
export function generateApiUrl({
  endpoint,
  locale,
  queryParams,
}: { endpoint: string; locale: Locale } & BaseRequestOptions) {
  const url = new URL(path.join(baseUrl, locale as string, endpoint));

  if (queryParams) {
    for (const [key, value] of Object.entries(queryParams)) {
      url.searchParams.set(key, value);
    }
  }

  return url;
}

interface ServerRequestOptions extends BaseRequestOptions {
  init?: RequestInit;
}

export interface BaseRequestOptions {
  queryParams?: Record<string, string>;
}
